/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "FrameSize.hh"
#include "framecpp/FrAdcData.hh"
#include "framecpp/FrVect.hh"
#include <string>

//====================================== Length of frame string structure
inline size_t
string_len(const std::string& s) {
  return 3 + s.size();
}

using namespace std;

size_t
FrameSize(const FrameCPP::FrameH& f) {
  //------------------------------------  File header length
  size_t sz = 40;
  sz += 515;   // FrSH lengths
  sz += 12156; // FrSE lengths
  
  //------------------------------------  Get FrameH length
  sz += 126 + string_len(f.GetName());

  //------------------------------------  FrHistory nd FrDetector
  sz += 95 + 62;

  //------------------------------------  Get raw data structure
  const auto& raw = f.GetRawData();
  sz += 48 + string_len(raw->GetName());

  //------------------------------------  Loop over FrAdcData
  size_t nFrame = 1;
  const auto& cadc = raw->RefFirstAdc();
  size_t toc_adc_data = 0;
  for (auto p=cadc.begin(); p != cadc.end(); p++) {
    size_t name_len = string_len((*p)->GetName());
    toc_adc_data += name_len + 8 + nFrame*8;
    sz += 86 + name_len + string_len((*p)->GetComment())
             + string_len((*p)->GetUnits()); // FrAdcData length
    auto vect((*p)->RefData().front());
    sz += 72 + string_len(vect->GetName()) + string_len(vect->GetUnitY())
             + vect->GetNBytes() + string_len(vect->GetDim(0).GetUnitX());
    size_t nAux = (*p)->RefAux().size();
    for (size_t i=0; i<nAux; ++i) {
      auto auxv((*p)->RefAux()[i]);
      if (auxv) {
	sz += 72 + string_len(auxv->GetName()) + string_len(auxv->GetUnitY())
	  + auxv->GetNBytes() + string_len(auxv->GetDim(0).GetUnitX());
      }
    }
  }
  sz += 34;  // FrEndOfFrame length
  
  //-------------------------------------  TOC Adc data length
  // cout << "estimated TOC ADC data length = " <<  toc_adc_data << endl;
  sz += 267 + toc_adc_data;

  sz += 46;  // FrEndOfFile length
  // cout << "estimated frame size = " << sz << endl;
  return sz;
}
